@@ -20,6 +20,7 @@ from string import Template
 from ferry.install import FERRY_HOME
 from ferry.docker.fabric import DockerFabric
 from ferry.config.hadoop.hiveconfig import *
+from ferry.config.system.info import *
 
 class HadoopClientInitializer(object):
     """
@@ -102,10 +103,20 @@ class HadoopClientInitializer(object):
         core_in_file.close()
         core_out_file.close()
 
-    """
-    Generate the core-site configuration. 
-    """
+    def _generate_log4j(self, new_config_dir):
+        in_file = open(self.template_dir + '/log4j.properties', 'r')
+        out_file = open(new_config_dir + '/log4j.properties', 'w+')
+
+        for line in in_file:
+            out_file.write(line)
+
+        in_file.close()
+        out_file.close()
+
     def _generate_core_site(self, hdfs_master, new_config_dir):
+        """
+        Generate the core-site configuration. 
+        """
         core_in_file = open(self.template_dir + '/core-site.xml.template', 'r')
         core_out_file = open(new_config_dir + '/core-site.xml', 'w+')
 
@@ -130,6 +141,13 @@ class HadoopClientInitializer(object):
 
         changes = { "YARN_MASTER":yarn_master,
                     "DATA_STAGING":"/service/data/client/staging" }
+
+        # Get memory information.
+        changes['MEM'] = get_total_memory()
+        changes['CMEM'] = max(get_total_memory() / 8, 512)
+        changes['RMEM'] = 2 * changes['CMEM']
+        changes['ROPTS'] = '-Xmx' + str(int(0.8 * changes['RMEM'])) + 'm'
+
         for line in yarn_in_file:
             s = Template(line).substitute(changes)
             yarn_out_file.write(s)
@@ -153,6 +171,13 @@ class HadoopClientInitializer(object):
                     "JOB_MAPS":1,
                     "HISTORY_SERVER":config.yarn_master, 
                     "DATA_TMP":"/service/data/client/tmp" }
+
+        # Get memory information.
+        changes['MMEM'] = max(get_total_memory() / 8, 512)
+        changes['RMEM'] = 2 * changes['MMEM']
+        changes['MOPTS'] = '-Xmx' + str(int(0.8 * changes['MMEM'])) + 'm'
+        changes['ROPTS'] = '-Xmx' + str(int(0.8 * changes['RMEM'])) + 'm'
+
         for line in mapred_in_file:
             s = Template(line).substitute(changes)
             mapred_out_file.write(s)
@@ -212,6 +237,7 @@ class HadoopClientInitializer(object):
 
         # Generate the Hadoop conf files.
         if config.yarn_master:
+            self._generate_log4j(new_config_dir)
             self._generate_mapred_site(config, containers, new_config_dir)
             self._generate_yarn_site(config.yarn_master, new_config_dir)
 
