@@ -4,23 +4,20 @@ import sys
 import logging
 from subprocess import Popen, PIPE
 
-def mkdir(directory):
-    if not os.path.isdir(directory):
-        cmd = 'mkdir -p %s' % directory
-        output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
-        logging.info(cmd)
-        logging.info(output)
-
 def mount(entry_point, mount_point):
     # Check if the mount point exists. If not
     # go ahead and create it. 
     cmd = 'mount -t glusterfs %s %s' % (entry_point,
                                         mount_point)
-    Popen(cmd, shell=True)
+    output = Popen(cmd, stdout=PIPE,shell=True).stdout.read()
+    logging.warning(output)
 
 def umount(mount_point):
-    cmd = 'umount %s' % mount_point
-    Popen(cmd, shell=True)
+    cmd = 'cat /etc/mtab | grep /service/data | awk \'{print $2}\''
+    output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
+    if output.strip() != "":
+        cmd = 'umount %s' % mount_point
+        Popen(cmd, shell=True)
 
 cmd = sys.argv[1]
 if cmd == "mount":
