@@ -33,7 +33,9 @@ class DockerInstance(object):
             self.internal_ip = None
             self.ports = {}
             self.image = ''
-            self.keys = None
+            self.keydir = None
+            self.keyname = None
+            self.privatekey = None
             self.volumes = None
             self.default_user = None
             self.name = None
@@ -49,7 +51,9 @@ class DockerInstance(object):
             self.default_user = json_data['user']
             self.name = json_data['name']
             self.args = json_data['args']
-            self.keys = json_data['keys']
+            self.keydir = json_data['keydir']
+            self.keyname =json_data['keyname']
+            self.privatekey = json_data['privatekey']
             self.volumes = json_data['volumes']
 
     """
@@ -64,7 +68,9 @@ class DockerInstance(object):
                        'container' : self.container,
                        'image' : self.image,
                        'type': self.service_type, 
-                       'keys' : self.keys,
+                       'keydir' : self.keydir,
+                       'keyname' : self.keyname, 
+                       'privatekey' : self.privatekey, 
                        'volumes' : self.volumes,
                        'user' : self.default_user,
                        'name' : self.name,
@@ -264,7 +270,7 @@ class DockerCLI(object):
     """
     Start a stopped container. 
     """
-    def start(self, container, service_type, keys, volumes, args):
+    def start(self, container, service_type, keydir, keyname, privatekey, volumes, args):
         cmd = self.docker + ' ' + self.start_cmd + ' ' + container
         logging.warning(cmd)
         output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
@@ -272,7 +278,9 @@ class DockerCLI(object):
         # Now parse the output to get the IP and port
         container = output.strip()
         return self.inspect(container = container, 
-                            keys = keys,
+                            keydir = keydir,
+                            keyname = keyname,
+                            privatekey = privatekey,
                             volumes = volumes,
                             service_type = service_type, 
                             args = args)
@@ -282,7 +290,7 @@ class DockerCLI(object):
     The Docker allocator will ignore subnet, volumes, instance_name, and key
     information since everything runs locally. 
     """
-    def run(self, service_type, image, volumes, keys, open_ports, host_map=None, expose_group=None, hostname=None, default_cmd=None, args=None, lxc_opts=None):
+    def run(self, service_type, image, volumes, keydir, keyname, privatekey, open_ports, host_map=None, expose_group=None, hostname=None, default_cmd=None, args=None, lxc_opts=None):
         flags = self.daemon 
 
         # Specify the hostname (this is optional)
@@ -303,11 +311,12 @@ class DockerCLI(object):
                 flags += ' %s:%s' % (v, volumes[v])
 
         # Add the key directory
-        if keys != None:
-            for v in keys.keys():
+        if keydir != None:
+            for v in keydir.keys():
                 flags += self.volume_flag
-                # flags += ' %s:%s' % (v, keys[v])
-                flags += ' %s:%s' % (keys[v], v)
+                flags += ' %s:%s' % (keydir[v], v)
+                flags += self.env_flag
+                flags += ' \"KEY=%s\"' % keyname
 
         # Add the lxc options
         if lxc_opts != None:
@@ -341,7 +350,7 @@ class DockerCLI(object):
             return None
 
         container = child.stdout.read().strip()
-        return self.inspect(container, keys, volumes, hostname, open_ports, host_map, service_type, args)
+        return self.inspect(container, keydir, keyname, privatekey, volumes, hostname, open_ports, host_map, service_type, args)
 
     def _get_lxc_net(self, lxc_tuples):
         for l in lxc_tuples:
@@ -354,7 +363,7 @@ class DockerCLI(object):
     Inspect a container and return information on how
     to connect to the container. 
     """
-    def inspect(self, container, keys=None, volumes=None, hostname=None, open_ports=[], host_map=None, service_type=None, args=None):
+    def inspect(self, container, keydir=None, keyname=None, privatekey=None, volumes=None, hostname=None, open_ports=[], host_map=None, service_type=None, args=None):
         cmd = self.docker + ' ' + self.inspect_cmd + ' ' + container
         logging.warning(cmd)
 
@@ -401,6 +410,8 @@ class DockerCLI(object):
         else:
             instance.volumes = data['Volumes']
 
-        if keys:
-            instance.keys = keys
+            
+        instance.keydir = keydir
+        instance.keyname = keyname
+        instance.privatekey = privatekey
         return instance
